from copy import deepcopy
from operator import itemgetter

row_sep = f"+{'-' * 32}+{'-' * 32}+"
justify = 30


def print_table(data, module_name):
    rows = {}
    for row in data:
        model_name = row["model_name"]
        try:
            rows[model_name].append([row["old_name"], row["new_name"]])
        except KeyError:
            rows[model_name] = []
            rows[model_name].append([row["old_name"], row["new_name"]])
    for model_name in rows:
        rows[model_name] = sorted(rows[model_name], key=itemgetter(0))

    for table_name, field_pair in rows.items():
        print("")
        print(f"Table: {module_name}_{table_name}")
        print(row_sep)
        print(f"| {'OLD NAME'.ljust(justify)} | {'NEW NAME'.ljust(justify)} |")
        print(row_sep)
        for old_name, new_name in field_pair:
            print(f"| {old_name.ljust(justify)} | {new_name.ljust(justify)} |")
            print(row_sep)


def fields_summary(data, fields=None):
    fields = fields or {}
    for row in data:
        field_name = row["old_name"]
        try:
            fields[field_name].append(row["new_name"])
        except KeyError:
            fields[field_name] = []
            fields[field_name].append(row["new_name"])
    for k, v in fields.items():
        fields[k] = list(set(v))
    return fields


def print_field_summary(fields):
    sorted_fields = {}
    keys = sorted(deepcopy(fields), key=itemgetter(0))
    for k in keys:
        sorted_fields.update({k: fields[k]})
    print("")
    print(row_sep)
    print(f"| {'OLD NAME'.ljust(justify)} | {'NEW NAME'.ljust(justify)} |")
    print(row_sep)
    for k, v in sorted_fields.items():
        print(f"| {k.ljust(justify)} | {v[0].ljust(justify)} |")
        print(row_sep)


meta_screening_data = [
    dict(
        model_name="historicalicpreferral",
        old_name="hba1c",
        new_name="hba1c_value",
    ),
    dict(
        model_name="historicalicpreferral",
        old_name="fasting_glucose",
        new_name="ifg_value",
    ),
    dict(
        model_name="historicalicpreferral",
        old_name="ogtt_two_hr",
        new_name="ogtt_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="calculated_bmi",
        new_name="calculated_bmi_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="calculated_egfr",
        new_name="calculated_egfr_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="converted_creatinine",
        new_name="converted_creatinine_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="converted_fasting_glucose",
        new_name="converted_ifg_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="converted_ogtt_two_hr",
        new_name="converted_ogtt_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="creatinine",
        new_name="creatinine_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="hba1c",
        new_name="hba1c_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="ogtt_two_hr_datetime",
        new_name="ogtt_datetime",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="ogtt_two_hr_quantifier",
        new_name="ogtt_quantifier",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="urine_bhcg",
        new_name="urine_bhcg_value",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="calculated_bmi",
        new_name="calculated_bmi_value",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="calculated_egfr",
        new_name="calculated_egfr_value",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="converted_creatinine",
        new_name="converted_creatinine_value",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="converted_fasting_glucose",
        new_name="converted_ifg_value",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="converted_ogtt_two_hr",
        new_name="converted_ogtt_value",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="creatinine",
        new_name="creatinine_value",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="hba1c",
        new_name="hba1c_value",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="ogtt_two_hr_datetime",
        new_name="ogtt_datetime",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="ogtt_two_hr_quantifier",
        new_name="ogtt_quantifier",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="ogtt_two_hr_units",
        new_name="ogtt_units",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="urine_bhcg",
        new_name="urine_bhcg_value",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="calculated_bmi",
        new_name="calculated_bmi_value",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="calculated_egfr",
        new_name="calculated_egfr_value",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="converted_creatinine",
        new_name="converted_creatinine_value",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="converted_fasting_glucose",
        new_name="converted_ifg_value",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="converted_ogtt_two_hr",
        new_name="converted_ogtt_value",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="creatinine",
        new_name="creatinine_value",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="hba1c",
        new_name="hba1c_value",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="ogtt_two_hr_datetime",
        new_name="ogtt_datetime",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="ogtt_two_hr_quantifier",
        new_name="ogtt_quantifier",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="ogtt_two_hr_units",
        new_name="ogtt_units",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="urine_bhcg",
        new_name="urine_bhcg_value",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="calculated_bmi",
        new_name="calculated_bmi_value",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="calculated_egfr",
        new_name="calculated_egfr_value",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="converted_creatinine",
        new_name="converted_creatinine_value",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="converted_fasting_glucose",
        new_name="converted_ifg_value",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="converted_ogtt_two_hr",
        new_name="converted_ogtt_value",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="creatinine",
        new_name="creatinine_value",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="hba1c",
        new_name="hba1c_value",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="ogtt_two_hr_datetime",
        new_name="ogtt_datetime",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="ogtt_two_hr_quantifier",
        new_name="ogtt_quantifier",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="ogtt_two_hr_units",
        new_name="ogtt_units",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="urine_bhcg",
        new_name="urine_bhcg_value",
    ),
    dict(
        model_name="icpreferral",
        old_name="hba1c",
        new_name="hba1c_value",
    ),
    dict(
        model_name="icpreferral",
        old_name="fasting_glucose",
        new_name="ifg_value",
    ),
    dict(
        model_name="icpreferral",
        old_name="ogtt_two_hr",
        new_name="ogtt_value",
    ),
    dict(
        model_name="subjectscreening",
        old_name="calculated_bmi",
        new_name="calculated_bmi_value",
    ),
    dict(
        model_name="subjectscreening",
        old_name="calculated_egfr",
        new_name="calculated_egfr_value",
    ),
    dict(
        model_name="subjectscreening",
        old_name="converted_creatinine",
        new_name="converted_creatinine_value",
    ),
    dict(
        model_name="subjectscreening",
        old_name="converted_fasting_glucose",
        new_name="converted_ifg_value",
    ),
    dict(
        model_name="subjectscreening",
        old_name="converted_ogtt_two_hr",
        new_name="converted_ogtt_value",
    ),
    dict(
        model_name="subjectscreening",
        old_name="creatinine",
        new_name="creatinine_value",
    ),
    dict(
        model_name="subjectscreening",
        old_name="hba1c",
        new_name="hba1c_value",
    ),
    dict(
        model_name="subjectscreening",
        old_name="ogtt_two_hr_datetime",
        new_name="ogtt_datetime",
    ),
    dict(
        model_name="subjectscreening",
        old_name="ogtt_two_hr_quantifier",
        new_name="ogtt_quantifier",
    ),
    dict(
        model_name="subjectscreening",
        old_name="ogtt_two_hr_units",
        new_name="ogtt_units",
    ),
    dict(
        model_name="subjectscreening",
        old_name="urine_bhcg",
        new_name="urine_bhcg_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="fasting_glucose",
        new_name="ifg_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="fasting_glucose_datetime",
        new_name="ifg_datetime",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="fasting_glucose_quantifier",
        new_name="ifg_quantifier",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="fasting_glucose_units",
        new_name="ifg_units",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="ogtt_two_hr",
        new_name="ogtt_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="ogtt_two_hr_units",
        new_name="ogtt_units",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="fasting_glucose",
        new_name="ifg_value",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="fasting_glucose_datetime",
        new_name="ifg_datetime",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="fasting_glucose_quantifier",
        new_name="ifg_quantifier",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="fasting_glucose_units",
        new_name="ifg_units",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="ogtt_two_hr",
        new_name="ogtt_value",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="fasting_glucose",
        new_name="ifg_value",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="fasting_glucose_datetime",
        new_name="ifg_datetime",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="fasting_glucose_quantifier",
        new_name="ifg_quantifier",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="fasting_glucose_units",
        new_name="ifg_units",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="ogtt_two_hr",
        new_name="ogtt_value",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="fasting_glucose",
        new_name="ifg_value",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="fasting_glucose_datetime",
        new_name="ifg_datetime",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="fasting_glucose_quantifier",
        new_name="ifg_quantifier",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="fasting_glucose_units",
        new_name="ifg_units",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="ogtt_two_hr",
        new_name="ogtt_value",
    ),
    dict(
        model_name="subjectscreening",
        old_name="fasting_glucose",
        new_name="ifg_value",
    ),
    dict(
        model_name="subjectscreening",
        old_name="fasting_glucose_datetime",
        new_name="ifg_datetime",
    ),
    dict(
        model_name="subjectscreening",
        old_name="fasting_glucose_quantifier",
        new_name="ifg_quantifier",
    ),
    dict(
        model_name="subjectscreening",
        old_name="fasting_glucose_units",
        new_name="ifg_units",
    ),
    dict(
        model_name="subjectscreening",
        old_name="ogtt_two_hr",
        new_name="ogtt_value",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="fasted",
        new_name="fasting",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="fasted_duration_minutes",
        new_name="fasting_duration_minutes",
    ),
    dict(
        model_name="historicalscreeningpartone",
        old_name="fasted_duration_str",
        new_name="fasting_duration_str",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="fasted",
        new_name="fasting",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="fasted_duration_minutes",
        new_name="fasting_duration_minutes",
    ),
    dict(
        model_name="historicalscreeningpartthree",
        old_name="fasted_duration_str",
        new_name="fasting_duration_str",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="fasted",
        new_name="fasting",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="fasted_duration_minutes",
        new_name="fasting_duration_minutes",
    ),
    dict(
        model_name="historicalscreeningparttwo",
        old_name="fasted_duration_str",
        new_name="fasting_duration_str",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="fasted",
        new_name="fasting",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="fasted_duration_minutes",
        new_name="fasting_duration_minutes",
    ),
    dict(
        model_name="historicalsubjectscreening",
        old_name="fasted_duration_str",
        new_name="fasting_duration_str",
    ),
    dict(
        model_name="subjectscreening",
        old_name="fasted",
        new_name="fasting",
    ),
    dict(
        model_name="subjectscreening",
        old_name="fasted_duration_minutes",
        new_name="fasting_duration_minutes",
    ),
    dict(
        model_name="subjectscreening",
        old_name="fasted_duration_str",
        new_name="fasting_duration_str",
    ),
]

meta_subject_data = [
    dict(
        model_name="historicalpatienthistory",
        old_name="hypertension_diagnosis",
        new_name="htn_diagnosis",
    ),
    dict(
        model_name="patienthistory",
        old_name="hypertension_diagnosis",
        new_name="htn_diagnosis",
    ),
    dict(
        model_name="bloodresultsfbc",
        old_name="fbc_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="bloodresultsfbc",
        old_name="haemoglobin",
        new_name="haemoglobin_value",
    ),
    dict(
        model_name="bloodresultsfbc",
        old_name="hct",
        new_name="hct_value",
    ),
    dict(
        model_name="bloodresultsfbc",
        old_name="platelets",
        new_name="platelets_value",
    ),
    dict(
        model_name="bloodresultsfbc",
        old_name="rbc",
        new_name="rbc_value",
    ),
    dict(
        model_name="bloodresultsfbc",
        old_name="wbc",
        new_name="wbc_value",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="glucose_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="glucose_abnormal",
        new_name="gluc_abnormal",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="glucose_quantifier",
        new_name="gluc_quantifier",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="glucose_reportable",
        new_name="gluc_reportable",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="glucose_units",
        new_name="gluc_units",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="glucose",
        new_name="gluc_value",
    ),
    dict(
        model_name="bloodresultshba1c",
        old_name="hba1c_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="bloodresultslft",
        old_name="albumin",
        new_name="albumin_value",
    ),
    dict(
        model_name="bloodresultslft",
        old_name="alp",
        new_name="alp_value",
    ),
    dict(
        model_name="bloodresultslft",
        old_name="alt",
        new_name="alt_value",
    ),
    dict(
        model_name="bloodresultslft",
        old_name="lft_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="bloodresultslft",
        old_name="ast",
        new_name="ast_value",
    ),
    dict(
        model_name="bloodresultslft",
        old_name="ggt",
        new_name="ggt_value",
    ),
    dict(
        model_name="bloodresultslipid",
        old_name="lipid_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="bloodresultsrft",
        old_name="rft_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="bloodresultsrft",
        old_name="egfr",
        new_name="egfr_value",
    ),
    dict(
        model_name="bloodresultsrft",
        old_name="urea",
        new_name="urea_value",
    ),
    dict(
        model_name="bloodresultsrft",
        old_name="uric_acid",
        new_name="uric_acid_value",
    ),
    dict(
        model_name="glucose",
        old_name="ogtt_two_hr_datetime",
        new_name="ogtt_datetime",
    ),
    dict(
        model_name="glucose",
        old_name="ogtt_two_hr_quantifier",
        new_name="ogtt_quantifier",
    ),
    dict(
        model_name="glucose",
        old_name="ogtt_two_hr_units",
        new_name="ogtt_units",
    ),
    dict(
        model_name="historicalbloodresultsfbc",
        old_name="fbc_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="historicalbloodresultsfbc",
        old_name="haemoglobin",
        new_name="haemoglobin_value",
    ),
    dict(
        model_name="historicalbloodresultsfbc",
        old_name="hct",
        new_name="hct_value",
    ),
    dict(
        model_name="historicalbloodresultsfbc",
        old_name="platelets",
        new_name="platelets_value",
    ),
    dict(
        model_name="historicalbloodresultsfbc",
        old_name="rbc",
        new_name="rbc_value",
    ),
    dict(
        model_name="historicalbloodresultsfbc",
        old_name="fbc_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="historicalbloodresultsfbc",
        old_name="wbc",
        new_name="wbc_value",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="glucose_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="glucose_abnormal",
        new_name="gluc_abnormal",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="glucose_quantifier",
        new_name="gluc_quantifier",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="glucose_reportable",
        new_name="gluc_reportable",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="glucose_units",
        new_name="gluc_units",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="glucose",
        new_name="gluc_value",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="glucose_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="historicalbloodresultshba1c",
        old_name="hba1c_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="historicalbloodresultshba1c",
        old_name="hba1c_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="historicalbloodresultslft",
        old_name="albumin",
        new_name="albumin_value",
    ),
    dict(
        model_name="historicalbloodresultslft",
        old_name="alp",
        new_name="alp_value",
    ),
    dict(
        model_name="historicalbloodresultslft",
        old_name="alt",
        new_name="alt_value",
    ),
    dict(
        model_name="historicalbloodresultslft",
        old_name="lft_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="historicalbloodresultslft",
        old_name="ast",
        new_name="ast_value",
    ),
    dict(
        model_name="historicalbloodresultslft",
        old_name="ggt",
        new_name="ggt_value",
    ),
    dict(
        model_name="historicalbloodresultslft",
        old_name="lft_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="historicalbloodresultslipid",
        old_name="lipid_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="historicalbloodresultslipid",
        old_name="lipid_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="historicalbloodresultsrft",
        old_name="rft_assay_datetime",
        new_name="assay_datetime",
    ),
    dict(
        model_name="historicalbloodresultsrft",
        old_name="egfr",
        new_name="egfr_value",
    ),
    dict(
        model_name="historicalbloodresultsrft",
        old_name="rft_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="historicalbloodresultsrft",
        old_name="urea",
        new_name="urea_value",
    ),
    dict(
        model_name="historicalbloodresultsrft",
        old_name="uric_acid",
        new_name="uric_acid_value",
    ),
    dict(
        model_name="historicalglucose",
        old_name="ogtt_two_hr_datetime",
        new_name="ogtt_datetime",
    ),
    dict(
        model_name="historicalglucose",
        old_name="ogtt_two_hr_quantifier",
        new_name="ogtt_quantifier",
    ),
    dict(
        model_name="historicalglucose",
        old_name="ogtt_two_hr_units",
        new_name="ogtt_units",
    ),
    #
    #
    #
    dict(
        model_name="bloodresultsfbc",
        old_name="fbc_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="glucose_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="bloodresultshba1c",
        old_name="hba1c",
        new_name="hba1c_value",
    ),
    dict(
        model_name="bloodresultshba1c",
        old_name="hba1c_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="bloodresultslft",
        old_name="amylase",
        new_name="amylase_value",
    ),
    dict(
        model_name="bloodresultslft",
        old_name="lft_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="bloodresultslipid",
        old_name="lipid_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="bloodresultsrft",
        old_name="creatinine",
        new_name="creatinine_value",
    ),
    dict(
        model_name="bloodresultsrft",
        old_name="rft_requisition",
        new_name="requisition",
    ),
    dict(
        model_name="glucose",
        old_name="fasting_glucose",
        new_name="ifg_value",
    ),
    dict(
        model_name="glucose",
        old_name="fasting_glucose_datetime",
        new_name="ifg_datetime",
    ),
    dict(
        model_name="glucose",
        old_name="fasting_glucose_quantifier",
        new_name="ifg_quantifier",
    ),
    dict(
        model_name="glucose",
        old_name="fasting_glucose_units",
        new_name="ifg_units",
    ),
    dict(
        model_name="glucose",
        old_name="ogtt_two_hr",
        new_name="ogtt_value",
    ),
    dict(
        model_name="historicalbloodresultshba1c",
        old_name="hba1c",
        new_name="hba1c_value",
    ),
    dict(
        model_name="historicalbloodresultslft",
        old_name="amylase",
        new_name="amylase_value",
    ),
    dict(
        model_name="historicalbloodresultsrft",
        old_name="creatinine",
        new_name="creatinine_value",
    ),
    dict(
        model_name="historicalglucose",
        old_name="fasting_glucose",
        new_name="ifg_value",
    ),
    dict(
        model_name="historicalglucose",
        old_name="fasting_glucose_datetime",
        new_name="ifg_datetime",
    ),
    dict(
        model_name="historicalglucose",
        old_name="fasting_glucose_quantifier",
        new_name="ifg_quantifier",
    ),
    dict(
        model_name="historicalglucose",
        old_name="fasting_glucose_units",
        new_name="ifg_units",
    ),
    dict(
        model_name="historicalglucose",
        old_name="ogtt_two_hr",
        new_name="ogtt_value",
    ),
    dict(
        model_name="bloodresultslipid",
        old_name="hdl",
        new_name="hdl_value",
    ),
    dict(
        model_name="bloodresultslipid",
        old_name="ldl",
        new_name="ldl_value",
    ),
    dict(
        model_name="bloodresultslipid",
        old_name="trig",
        new_name="trig_value",
    ),
    dict(
        model_name="historicalbloodresultslipid",
        old_name="hdl",
        new_name="hdl_value",
    ),
    dict(
        model_name="historicalbloodresultslipid",
        old_name="ldl",
        new_name="ldl_value",
    ),
    dict(
        model_name="historicalbloodresultslipid",
        old_name="trig",
        new_name="trig_value",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="gluc_abnormal",
        new_name="glucose_abnormal",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="gluc_quantifier",
        new_name="glucose_quantifier",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="gluc_reportable",
        new_name="glucose_reportable",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="gluc_units",
        new_name="glucose_units",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="gluc_abnormal",
        new_name="glucose_abnormal",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="gluc_quantifier",
        new_name="glucose_quantifier",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="gluc_reportable",
        new_name="glucose_reportable",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="gluc_units",
        new_name="glucose_units",
    ),
    dict(
        model_name="bloodresultsglu",
        old_name="gluc_value",
        new_name="glucose_value",
    ),
    dict(
        model_name="historicalbloodresultsglu",
        old_name="gluc_value",
        new_name="glucose_value",
    ),
    dict(
        model_name="glucose",
        old_name="fasted",
        new_name="fasting",
    ),
    dict(
        model_name="glucose",
        old_name="fasted_duration_minutes",
        new_name="fasting_duration_minutes",
    ),
    dict(
        model_name="glucose",
        old_name="fasted_duration_str",
        new_name="fasting_duration_str",
    ),
    dict(
        model_name="historicalglucose",
        old_name="fasted",
        new_name="fasting",
    ),
    dict(
        model_name="historicalglucose",
        old_name="fasted_duration_minutes",
        new_name="fasting_duration_minutes",
    ),
    dict(
        model_name="historicalglucose",
        old_name="fasted_duration_str",
        new_name="fasting_duration_str",
    ),
]

# print_table(meta_screening_data, "meta_screening")
# print_table(meta_subject_data, "meta_subject")
# meta_fields = fields_summary(meta_screening_data)
# meta_fields = fields_summary(meta_screening_data, fields=meta_fields)
# print_field_summary(meta_fields)



# Generated by Django 3.2.4 on 2021-06-28 18:05

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0020_auto_20200524_1944"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalicpreferral",
            old_name="hba1c",
            new_name="hba1c_value",
        ),
        migrations.RenameField(
            model_name="historicalicpreferral",
            old_name="fasting_glucose",
            new_name="ifg_value",
        ),
        migrations.RenameField(
            model_name="historicalicpreferral",
            old_name="ogtt_two_hr",
            new_name="ogtt_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="calculated_bmi",
            new_name="calculated_bmi_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="calculated_egfr",
            new_name="calculated_egfr_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="converted_creatinine",
            new_name="converted_creatinine_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="converted_fasting_glucose",
            new_name="converted_ifg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="converted_ogtt_two_hr",
            new_name="converted_ogtt_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="creatinine",
            new_name="creatinine_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="hba1c",
            new_name="hba1c_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ogtt_two_hr_datetime",
            new_name="ogtt_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ogtt_two_hr_quantifier",
            new_name="ogtt_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="urine_bhcg",
            new_name="urine_bhcg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="calculated_bmi",
            new_name="calculated_bmi_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="calculated_egfr",
            new_name="calculated_egfr_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="converted_creatinine",
            new_name="converted_creatinine_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="converted_fasting_glucose",
            new_name="converted_ifg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="converted_ogtt_two_hr",
            new_name="converted_ogtt_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="creatinine",
            new_name="creatinine_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="hba1c",
            new_name="hba1c_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ogtt_two_hr_datetime",
            new_name="ogtt_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ogtt_two_hr_quantifier",
            new_name="ogtt_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ogtt_two_hr_units",
            new_name="ogtt_units",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="urine_bhcg",
            new_name="urine_bhcg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="calculated_bmi",
            new_name="calculated_bmi_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="calculated_egfr",
            new_name="calculated_egfr_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="converted_creatinine",
            new_name="converted_creatinine_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="converted_fasting_glucose",
            new_name="converted_ifg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="converted_ogtt_two_hr",
            new_name="converted_ogtt_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="creatinine",
            new_name="creatinine_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="hba1c",
            new_name="hba1c_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ogtt_two_hr_datetime",
            new_name="ogtt_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ogtt_two_hr_quantifier",
            new_name="ogtt_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ogtt_two_hr_units",
            new_name="ogtt_units",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="urine_bhcg",
            new_name="urine_bhcg_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="calculated_bmi",
            new_name="calculated_bmi_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="calculated_egfr",
            new_name="calculated_egfr_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="converted_creatinine",
            new_name="converted_creatinine_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="converted_fasting_glucose",
            new_name="converted_ifg_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="converted_ogtt_two_hr",
            new_name="converted_ogtt_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="creatinine",
            new_name="creatinine_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="hba1c",
            new_name="hba1c_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ogtt_two_hr_datetime",
            new_name="ogtt_datetime",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ogtt_two_hr_quantifier",
            new_name="ogtt_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ogtt_two_hr_units",
            new_name="ogtt_units",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="urine_bhcg",
            new_name="urine_bhcg_value",
        ),
        migrations.RenameField(
            model_name="icpreferral",
            old_name="hba1c",
            new_name="hba1c_value",
        ),
        migrations.RenameField(
            model_name="icpreferral",
            old_name="fasting_glucose",
            new_name="ifg_value",
        ),
        migrations.RenameField(
            model_name="icpreferral",
            old_name="ogtt_two_hr",
            new_name="ogtt_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="calculated_bmi",
            new_name="calculated_bmi_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="calculated_egfr",
            new_name="calculated_egfr_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="converted_creatinine",
            new_name="converted_creatinine_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="converted_fasting_glucose",
            new_name="converted_ifg_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="converted_ogtt_two_hr",
            new_name="converted_ogtt_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="creatinine",
            new_name="creatinine_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="hba1c",
            new_name="hba1c_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ogtt_two_hr_datetime",
            new_name="ogtt_datetime",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ogtt_two_hr_quantifier",
            new_name="ogtt_quantifier",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ogtt_two_hr_units",
            new_name="ogtt_units",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="urine_bhcg",
            new_name="urine_bhcg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="fasting_glucose",
            new_name="ifg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="fasting_glucose_datetime",
            new_name="ifg_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="fasting_glucose_quantifier",
            new_name="ifg_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="fasting_glucose_units",
            new_name="ifg_units",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ogtt_two_hr",
            new_name="ogtt_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ogtt_two_hr_units",
            new_name="ogtt_units",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="fasting_glucose",
            new_name="ifg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="fasting_glucose_datetime",
            new_name="ifg_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="fasting_glucose_quantifier",
            new_name="ifg_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="fasting_glucose_units",
            new_name="ifg_units",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ogtt_two_hr",
            new_name="ogtt_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="fasting_glucose",
            new_name="ifg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="fasting_glucose_datetime",
            new_name="ifg_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="fasting_glucose_quantifier",
            new_name="ifg_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="fasting_glucose_units",
            new_name="ifg_units",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ogtt_two_hr",
            new_name="ogtt_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="fasting_glucose",
            new_name="ifg_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="fasting_glucose_datetime",
            new_name="ifg_datetime",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="fasting_glucose_quantifier",
            new_name="ifg_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="fasting_glucose_units",
            new_name="ifg_units",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ogtt_two_hr",
            new_name="ogtt_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="fasting_glucose",
            new_name="ifg_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="fasting_glucose_datetime",
            new_name="ifg_datetime",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="fasting_glucose_quantifier",
            new_name="ifg_quantifier",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="fasting_glucose_units",
            new_name="ifg_units",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ogtt_two_hr",
            new_name="ogtt_value",
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ifg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="<u>Time</u> IFG level measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ifg_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                verbose_name="IFG quantifier",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ifg_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="IFG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ifg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="IFG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ifg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="<u>Time</u> IFG level measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ifg_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                verbose_name="IFG quantifier",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ifg_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="IFG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ifg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="IFG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ifg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="<u>Time</u> IFG level measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ifg_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                verbose_name="IFG quantifier",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ifg_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="IFG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ifg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="IFG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ifg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="<u>Time</u> IFG level measured"
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ifg_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                verbose_name="IFG quantifier",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ifg_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="IFG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ifg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="IFG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ifg_datetime",
            field=models.DateTimeField(
                blank=True, null=True, verbose_name="<u>Time</u> IFG level measured"
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ifg_quantifier",
            field=models.CharField(
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                verbose_name="IFG quantifier",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ifg_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="IFG units",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ifg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="IFG level",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="Blood glucose <u>level</u> 2-hours after oral glucose solution given",
            ),
        ),
    ]



# Generated by Django 4.2.5 on 2023-10-17 22:25

import _socket
import django_audit_fields.fields.hostname_modification_field
import edc_action_item.managers
import edc_identifier.managers
import edc_sites.models
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "meta_prn",
            "0052_alter_historicalprotocolincident_reasons_withdrawn_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="losstofollowup",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Loss to Follow Up",
                "verbose_name_plural": "Loss to Follow Up",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedule",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Off-schedule",
                "verbose_name_plural": "Off-schedule",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedulepostnatal",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Off-schedule: post-natal",
                "verbose_name_plural": "Off-schedule: post-natal",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedulepregnancy",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Off-schedule: Pregnancy",
                "verbose_name_plural": "Off-schedule: Pregnancy",
            },
        ),
        migrations.AlterModelOptions(
            name="offstudymedication",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Withdrawal of Study Drug",
                "verbose_name_plural": "Withdrawal of Study Drug",
            },
        ),
        migrations.AlterModelOptions(
            name="onschedule",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
            },
        ),
        migrations.AlterModelOptions(
            name="onschedulepregnancy",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
            },
        ),
        migrations.AlterModelOptions(
            name="pregnancynotification",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Pregnancy Notification",
                "verbose_name_plural": "Pregnancy Notifications",
            },
        ),
        migrations.AlterModelOptions(
            name="protocolincident",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Protocol Incident",
                "verbose_name_plural": "Protocol Incident",
            },
        ),
        migrations.AlterModelOptions(
            name="subjecttransfer",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Subject Transfer",
                "verbose_name_plural": "Subject Transfers",
            },
        ),
        migrations.AlterModelManagers(
            name="endofstudy",
            managers=[
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="losstofollowup",
            managers=[
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="onschedule",
            managers=[
                ("objects", edc_identifier.managers.SubjectIdentifierManager()),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="onschedulepostnatal",
            managers=[
                ("objects", edc_identifier.managers.SubjectIdentifierManager()),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="onschedulepregnancy",
            managers=[
                ("objects", edc_identifier.managers.SubjectIdentifierManager()),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedule",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalonschedule",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalonschedule",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedule",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepostnatal",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepostnatal",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepostnatal",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepostnatal",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepregnancy",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepregnancy",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepregnancy",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepregnancy",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepostnatal",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="onschedulepostnatal",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="onschedulepostnatal",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepostnatal",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepregnancy",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="onschedulepregnancy",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="onschedulepregnancy",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepregnancy",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
    ]



from edc_registration.models import RegisteredSubject

from meta_subject.models import Glucose


def get_sequential_fgb():
    """look for anyone with two or more sequential FBG measurements"""

    for rs in RegisteredSubject.objects.all().order_by("subject_identifier"):
        values = []
        for index, glucose in enumerate(
            Glucose.objects.filter(
                subject_visit__subject_identifier=rs.subject_identifier
            ).order_by("report_datetime")
        ):
            values.append(glucose.fbg_value)



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_qareports.modeladmin_mixins import NoteModelAdminMixin

from ....admin_site import meta_reports_admin
from ....forms import MissingOgttNoteForm
from ....models import MissingOgttNote


@admin.register(MissingOgttNote, site=meta_reports_admin)
class MissingOgttNoteModelAdmin(
    NoteModelAdminMixin,
    admin.ModelAdmin,
):
    """A modeladmin class for the Note model."""

    form = MissingOgttNoteForm
    note_template_name = "edc_qareports/qa_report_note.html"

    fieldsets = (
        (
            None,
            {"fields": ("subject_identifier", "report_datetime", "result_status")},
        ),
        (
            "OGTT",
            {
                "fields": (
                    "fasting",
                    "ogtt_base_datetime",
                    "ogtt_datetime",
                    "ogtt_value",
                    "ogtt_units",
                )
            },
        ),
        (
            "Note",
            {
                "fields": (
                    "note",
                    "report_model",
                )
            },
        ),
        audit_fieldset_tuple,
    )

    radio_fields = {
        "ogtt_units": admin.VERTICAL,
        "result_status": admin.VERTICAL,
        "fasting": admin.VERTICAL,
    }



import numpy as np
import pandas as pd
from django_pandas.io import read_frame
from edc_appointment.constants import MISSED_APPT  # noqa
from edc_pdutils.dataframes import get_eos, get_subject_consent, get_subject_visit

from meta_subject.models import Glucose, GlucoseFbg


def get_glucose_df() -> pd.DataFrame:
    subject_visit_df = (
        get_subject_visit("meta_subject.subjectvisit")
        .rename(columns={"id": "subject_visit_id"})
        .query("appt_timing!=@MISSED_APPT")
    )
    df_glucose_fbg = read_frame(GlucoseFbg.objects.all(), verbose=False).rename(
        columns={"fasting": "fasted", "subject_visit": "subject_visit_id"},
    )
    df_glucose_fbg["fasting_hrs"] = np.nan
    df_glucose_fbg["fasting_hrs"] = df_glucose_fbg["fasting_duration_delta"].apply(
        lambda x: x.total_seconds() / 3600
    )
    df_glucose_fbg.loc[
        :,
        ["ogtt_value", "ogtt_units", "ogtt_datetime"],
    ] = [np.nan, None, pd.NaT]
    df_glucose_fbg["source"] = "meta_subject.glucosefbg"
    df_glucose_fbg = pd.merge(
        subject_visit_df[
            [
                "subject_identifier",
                "site_id",
                "visit_code",
                "visit_datetime",
                "baseline_datetime",
                "subject_visit_id",
            ]
        ],
        df_glucose_fbg[[col for col in df_glucose_fbg.columns if "site_id" not in col]],
        on="subject_visit_id",
        how="left",
    )

    df_glucose = read_frame(Glucose.objects.all(), verbose=False).rename(
        columns={"subject_visit": "subject_visit_id", "fasting": "fasted"}
    )
    df_glucose["fasting_hrs"] = np.nan
    df_glucose["fasting_hrs"] = df_glucose["fasting_duration_delta"].apply(
        lambda x: x.total_seconds() / 3600
    )
    df_glucose["source"] = "meta_subject.glucose"

    df_glucose = pd.merge(
        subject_visit_df[
            [
                "subject_identifier",
                "site_id",
                "visit_code",
                "visit_datetime",
                "baseline_datetime",
                "subject_visit_id",
            ]
        ],
        df_glucose[[col for col in df_glucose.columns if "site_id" not in col]],
        on="subject_visit_id",
        how="left",
    )

    keep_cols = [
        "subject_identifier",
        "site_id",
        "visit_code",
        "visit_datetime",
        "baseline_datetime",
        "subject_visit_id",
        "fasted",
        "fasting_hrs",
        "fbg_value",
        "fbg_units",
        "fbg_datetime",
        "ogtt_value",
        "ogtt_units",
        "ogtt_datetime",
        "source",
        "revision",
        "report_datetime",
    ]
    df = pd.merge(
        df_glucose[keep_cols],
        df_glucose_fbg[keep_cols],
        on="subject_visit_id",
        how="outer",
        # indicator=True,
        suffixes=("", "_2"),
    )

    for suffix in ["", "_2"]:
        df[[f"fasting_hrs{suffix}", f"fbg_value{suffix}", f"ogtt_value{suffix}"]] = df[
            [f"fasting_hrs{suffix}", f"fbg_value{suffix}", f"ogtt_value{suffix}"]
        ].apply(pd.to_numeric)
        df.loc[
            (df[f"fbg_units{suffix}"] != "mmol/L (millimoles/L)")
            & (df[f"fbg_value{suffix}"] >= 0),
            f"fbg_units{suffix}",
        ] = "mmol/L (millimoles/L)"
        df.loc[
            (df[f"ogtt_units{suffix}"] != "mmol/L (millimoles/L)")
            & (df[f"ogtt_value{suffix}"] >= 0),
            f"ogtt_units{suffix}",
        ] = "mmol/L (millimoles/L)"

    # reconcile all to single column
    for col in ["fasted", "fbg_value", "ogtt_value", "fbg_datetime", "ogtt_datetime"]:
        df.loc[(df[col].isna()) & (df[f"{col}_2"].notna()), col] = df[f"{col}_2"]

    df_consent = get_subject_consent("meta_consent.subjectconsent")
    df_eos = get_eos("meta_prn.endofstudy")
    df = df.merge(
        df_consent[["subject_identifier", "gender", "consent_datetime", "dob"]],
        on="subject_identifier",
        how="left",
    ).merge(
        df_eos[["subject_identifier", "offstudy_datetime", "offstudy_reason"]],
        on="subject_identifier",
        how="left",
    )

    df[[col for col in df.columns if "datetime" in col]] = df[
        [col for col in df.columns if "datetime" in col]
    ].apply(lambda x: x.dt.tz_localize(None) if x.dtype == "datetime64[ns, UTC]" else x)

    df["visit_days"] = df["baseline_datetime"].rsub(df["visit_datetime"]).dt.days
    df["fgb_days"] = df["baseline_datetime"].rsub(df["fbg_datetime"]).dt.days
    df["ogtt_days"] = df["baseline_datetime"].rsub(df["ogtt_datetime"]).dt.days
    df["visit_days"] = pd.to_numeric(df["visit_days"], downcast="integer")
    df["fgb_days"] = pd.to_numeric(df["fgb_days"], downcast="integer")
    df["ogtt_days"] = pd.to_numeric(df["ogtt_days"], downcast="integer")

    df = (
        df.query(
            "offstudy_reason != 'Patient fulfilled late exclusion criteria "
            "(due to abnormal blood values or raised blood pressure at enrolment'"
        )
        .copy()
        .drop(columns=[col for col in df.columns if "_2" in col])
        .sort_values(by=["subject_identifier", "visit_code"])
        .reset_index(drop=True)
    )
    return df



# Generated by Django 3.2.13 on 2022-07-04 15:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0052_alter_historicalscreeningpartone_p3_ltfu_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalicpreferral",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical icp referral",
                "verbose_name_plural": "historical icp referrals",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalscreeningpartone",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Subject Screening: Part One",
                "verbose_name_plural": "historical Subject Screening: Part One",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalscreeningpartthree",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Subject Screening: Part Three",
                "verbose_name_plural": "historical Subject Screening: Part Three",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalscreeningparttwo",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Subject Screening: Part Two",
                "verbose_name_plural": "historical Subject Screening: Part Two",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalsubjectrefusal",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Subject Refusal",
                "verbose_name_plural": "historical Subject Refusals",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalsubjectscreening",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Subject Screening",
                "verbose_name_plural": "historical Subject Screening",
            },
        ),
        migrations.AlterField(
            model_name="historicalicpreferral",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalsubjectrefusal",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
    ]



from django.contrib import admin
from django.urls.conf import include, path

from meta_ae.admin_site import meta_ae_admin

urlpatterns = [
    path("ae/", include("meta_ae.urls")),
    path("admin/", meta_ae_admin.urls),
    path("admin/", admin.site.urls),
]



from django.contrib import admin
from django.utils.translation import gettext as _
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset
from edc_model_admin.history import SimpleHistoryAdmin

from ...admin_site import meta_subject_admin
from ...forms import HealthEconomicsUpdateForm
from ...models import HealthEconomicsUpdate
from ..modeladmin import CrfModelAdminMixin


@admin.register(HealthEconomicsUpdate, site=meta_subject_admin)
class HealthEconomicsUpdateAdmin(CrfModelAdminMixin, SimpleHistoryAdmin):
    form = HealthEconomicsUpdateForm

    additional_instructions = _(
        """We want to learn about the household and we use these
        questions to get an understanding of wealth and opportunities
        in the community. A person or persons (people/ members) who share
        the same kitchen (pot), live together, and run the household
        expenditure from the same income is known as a 'household'. Household
        members should be identified on the basis that they shared a place of
        living together most of time for the past one year. When it is
        difficult to demarcate 'most of the time', living together for the
        past six months or more should be used to find out whether or not
        the person is a household member. For polygamous households, the
        household head should think about all the households and respond
        with respect to all households."""
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_visit",
                    "report_datetime",
                )
            },
        ),
        (
            "A: Household and head of household",
            {
                "fields": (
                    "hh_count",
                    "hh_minors_count",
                    "hoh",
                    "relationship_to_hoh",
                    "relationship_to_hoh_other",
                ),
            },
        ),
        (
            "B: Rooms and beds in your household",
            {
                "description": _(
                    "Before asking about household income, we would first like"
                    "to know about the number of rooms in the home and number of beds. "
                ),
                "fields": (
                    "rooms",
                    "bedrooms",
                    "beds",
                ),
            },
        ),
        (
            "C: Household income",
            {
                "description": _(
                    "Now, I will ask about income for the household from paid work"
                    "or other sources. Please think of the total income for the "
                    "household, including income from all household members "
                    "as defined above. I know it may be difficult to estimate, "
                    "but please do try to give amounts as accurately as possible. "
                    "Please note that all information will be kept strictly "
                    "confidential. This information is important to assess overall "
                    "health and well-being of people in your household, compared "
                    "to other similar households."
                ),
                "fields": (
                    "avg_income",
                    "avg_income_value_known",
                    "avg_income_value",
                    "external_dependents",
                    "financial_status",
                ),
            },
        ),
        crf_status_fieldset,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "hoh": admin.VERTICAL,
        "relationship_to_hoh": admin.VERTICAL,
        "avg_income": admin.VERTICAL,
        "avg_income_value_known": admin.VERTICAL,
        "financial_status": admin.VERTICAL,
    }



# Generated by Django 5.1.2 on 2025-01-15 17:25

from django.db import IntegrityError, migrations, transaction
from edc_constants.constants import OTHER
from edc_utils import get_utcnow

from meta_lists.models import OffstudyReasons
from meta_prn.action_items import OffStudyMedicationAction
from meta_prn.models import EndOfStudy


def update_offstudy_reason(apps, schema_editor):
    """Datafix: Correct offstudy_reason from `other` to 'diabetes'
    based on the `other_offstudy_reason`. These four subjects were
    taken off study before 'diabetes' was an option.

    Create missing OffStudyMedicationAction if needed.
    """
    subject_identifiers = [
        "105-60-0029-0",
        "105-60-0106-6",
        "105-60-0145-4",
        "105-60-0166-0",
    ]
    other_reason_obj = OffstudyReasons.objects.get(name=OTHER)
    diabetes_reason_obj = OffstudyReasons.objects.get(name="diabetes")
    for obj in EndOfStudy.objects.filter(
        subject_identifier__in=subject_identifiers,
        offstudy_reason=other_reason_obj,
        other_offstudy_reason__icontains="THE CLIENT HAS DEVELOPED DIABETES",
    ):
        try:
            with transaction.atomic():
                OffStudyMedicationAction(
                    subject_identifier=obj.subject_identifier,
                    skip_get_current_site=True,
                    site_id=obj.site_id,
                )
        except IntegrityError:
            pass

        obj.offstudy_reason = diabetes_reason_obj
        obj.comment = obj.other_offstudy_reason
        obj.comment = obj.comment + " [datafix: 0061_auto_20250115_2025]"
        obj.other_offstudy_reason = ""
        obj.modified = get_utcnow()
        obj.user_modified = "erikvw"
        obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0060_alter_onschedule_managers_and_more"),
    ]

    operations = [migrations.RunPython(update_offstudy_reason)]



# Generated by Django 5.0 on 2024-01-09 23:21

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_action_item",
            "0035_alter_actionitem_options_alter_actiontype_options_and_more",
        ),
        ("edc_pharmacy", "0021_alter_box_options_alter_container_options_and_more"),
        ("edc_protocol_incident", "0021_alter_actionsrequired_options_and_more"),
        (
            "meta_lists",
            "0016_alter_abnormalfootappearanceobservations_options_and_more",
        ),
        ("meta_prn", "0053_alter_losstofollowup_options_and_more"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="losstofollowup",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Loss to Follow Up",
                "verbose_name_plural": "Loss to Follow Up",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedule",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Off-schedule",
                "verbose_name_plural": "Off-schedule",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedulepostnatal",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Off-schedule: post-natal",
                "verbose_name_plural": "Off-schedule: post-natal",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedulepregnancy",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Off-schedule: Pregnancy",
                "verbose_name_plural": "Off-schedule: Pregnancy",
            },
        ),
        migrations.AlterModelOptions(
            name="offstudymedication",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Withdrawal of Study Drug",
                "verbose_name_plural": "Withdrawal of Study Drug",
            },
        ),
        migrations.AlterModelOptions(
            name="onschedule",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
        ),
        migrations.AlterModelOptions(
            name="onschedulepregnancy",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
        ),
        migrations.AlterModelOptions(
            name="pregnancynotification",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Pregnancy Notification",
                "verbose_name_plural": "Pregnancy Notifications",
            },
        ),
        migrations.AlterModelOptions(
            name="protocolincident",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Protocol Incident",
                "verbose_name_plural": "Protocol Incident",
            },
        ),
        migrations.AlterModelOptions(
            name="subjecttransfer",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Subject Transfer",
                "verbose_name_plural": "Subject Transfers",
            },
        ),
        migrations.RemoveIndex(
            model_name="offschedule",
            name="meta_prn_of_id_d32eac_idx",
        ),
        migrations.RemoveIndex(
            model_name="offschedulepostnatal",
            name="meta_prn_of_id_3c0260_idx",
        ),
        migrations.RemoveIndex(
            model_name="offschedulepregnancy",
            name="meta_prn_of_id_45b8b5_idx",
        ),
        migrations.RemoveIndex(
            model_name="onschedule",
            name="meta_prn_on_id_6e5134_idx",
        ),
        migrations.RemoveIndex(
            model_name="onschedulepostnatal",
            name="meta_prn_on_id_e5ef52_idx",
        ),
        migrations.RemoveIndex(
            model_name="onschedulepregnancy",
            name="meta_prn_on_id_f00151_idx",
        ),
        migrations.RemoveIndex(
            model_name="protocolincident",
            name="meta_prn_pr_subject_e85b31_idx",
        ),
        migrations.AddField(
            model_name="endofstudy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="endofstudy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicallosstofollowup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicallosstofollowup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffschedule",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffschedule",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffschedulepostnatal",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffschedulepostnatal",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffschedulepregnancy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffschedulepregnancy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffstudymedication",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffstudymedication",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalonschedule",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalonschedule",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalonschedulepostnatal",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalonschedulepostnatal",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalonschedulepregnancy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalonschedulepregnancy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalpregnancynotification",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalpregnancynotification",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalprotocolincident",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalprotocolincident",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjecttransfer",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjecttransfer",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="losstofollowup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="losstofollowup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="offschedule",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="offschedule",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="offschedulepostnatal",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="offschedulepostnatal",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="offschedulepregnancy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="offschedulepregnancy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="offstudymedication",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="offstudymedication",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="onschedule",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="onschedule",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="onschedulepostnatal",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="onschedulepostnatal",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="onschedulepregnancy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="onschedulepregnancy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="pregnancynotification",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="pregnancynotification",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="protocolincident",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="protocolincident",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="subjecttransfer",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="subjecttransfer",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddIndex(
            model_name="offschedule",
            index=models.Index(
                fields=["subject_identifier", "offschedule_datetime", "site"],
                name="meta_prn_of_subject_5df032_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="offschedulepostnatal",
            index=models.Index(
                fields=["subject_identifier", "offschedule_datetime", "site"],
                name="meta_prn_of_subject_95953b_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="offschedulepregnancy",
            index=models.Index(
                fields=["subject_identifier", "offschedule_datetime", "site"],
                name="meta_prn_of_subject_51d402_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="offstudymedication",
            index=models.Index(
                fields=["modified", "created"], name="meta_prn_of_modifie_1b436a_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="offstudymedication",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="meta_prn_of_user_mo_63715c_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="onschedule",
            index=models.Index(
                fields=["subject_identifier", "onschedule_datetime", "site"],
                name="meta_prn_on_subject_69498c_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="onschedulepostnatal",
            index=models.Index(
                fields=["subject_identifier", "onschedule_datetime", "site"],
                name="meta_prn_on_subject_d31aa5_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="onschedulepregnancy",
            index=models.Index(
                fields=["subject_identifier", "onschedule_datetime", "site"],
                name="meta_prn_on_subject_f42322_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="pregnancynotification",
            index=models.Index(
                fields=["modified", "created"], name="meta_prn_pr_modifie_0d3c91_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="pregnancynotification",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="meta_prn_pr_user_mo_9dd482_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="protocolincident",
            index=models.Index(
                fields=["subject_identifier", "action_identifier", "site"],
                name="meta_prn_pr_subject_3281d1_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="subjecttransfer",
            index=models.Index(
                fields=[
                    "action_identifier",
                    "action_item",
                    "related_action_item",
                    "parent_action_item",
                ],
                name="meta_prn_su_action__e17e6d_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="subjecttransfer",
            index=models.Index(
                fields=[
                    "subject_identifier",
                    "action_identifier",
                    "report_datetime",
                    "site",
                ],
                name="meta_prn_su_subject_d9163a_idx",
            ),
        ),
    ]



# Generated by Django 4.2.11 on 2024-04-04 16:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0057_historicalonscheduledmreferral_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="dmreferral",
            name="referral_note",
            field=models.TextField(
                null=True,
                verbose_name="Please provide a brief history of the diabetes diagnosis that lead to this referral",
            ),
        ),
        migrations.AddField(
            model_name="historicaldmreferral",
            name="referral_note",
            field=models.TextField(
                null=True,
                verbose_name="Please provide a brief history of the diabetes diagnosis that lead to this referral",
            ),
        ),
    ]



import numpy as np
import pandas as pd
from django.apps import apps as django_apps
from edc_constants.constants import YES
from edc_utils import get_utcnow

from ..constants import (
    CASE_EOS,
    CASE_FBGS_WITH_FIRST_OGTT,
    CASE_FBGS_WITH_SECOND_OGTT,
    CASE_OGTT,
    endpoint_cases,
    endpoint_columns,
)
from ..get_glucose_df import get_glucose_df
from ..utils import (
    get_empty_endpoint_df,
    get_test_string,
    get_unique_subject_identifiers,
)
from .endpoint_by_date import EndpointByDate


class GlucoseEndpointsByDate:
    """
    Usage:
        cls = GlucoseEndpointsByDate()
        cls.run()

        # subjects who reached endpoint
        cls.endpoint_only_df.endpoint_type.value_counts()
        cls.endpoint_only_df.endpoint_label.value_counts()

        # subjects who reached endpoint with total
        result_df = cls.endpoint_only_df.endpoint_label.value_counts().to_frame().reset_index()
        result_df.columns = ["endpoint_label", "total"]
        result_df.loc[-1] = ["total", result_df.total.sum()]
        result_df = result_df.reset_index(drop=True)
        result_df
    """

    fbg_threshhold = 7.0
    ogtt_threshhold = 11.1
    endpoint_cls = EndpointByDate
    keep_cols = [
        "fasted",
        "fasting_hrs",
        "fbg_value",
        "fbg_units",
        "fbg_datetime",
        "ogtt_value",
        "ogtt_units",
        "ogtt_datetime",
        "source",
        "report_datetime",
        "subject_visit_id",
        "subject_identifier",
        "visit_code",
        "visit_datetime",
        "site_id",
        "baseline_datetime",
    ]

    def __init__(
        self, subject_identifiers: list[str] | None = None, case_list: list[int] | None = None
    ):

        self._glucose_fbg_df = pd.DataFrame()
        self._glucose_fbg_ogtt_df = pd.DataFrame()
        self.endpoint_only_df = pd.DataFrame()

        self.subject_identifiers = subject_identifiers or []
        self.case_list = case_list or [
            CASE_OGTT,
            CASE_FBGS_WITH_FIRST_OGTT,
            CASE_FBGS_WITH_SECOND_OGTT,
            CASE_EOS,
        ]
        self.endpoint_cases = {k: v for k, v in endpoint_cases.items() if k in self.case_list}

        self.df = get_glucose_df().copy()

        # label rows by type of glu tests (ones with value)
        self.df["test"] = self.df.apply(get_test_string, axis=1)

        self.df = self.df.sort_values(by=["subject_identifier", "fbg_datetime"]).reset_index(
            drop=True
        )
        self.working_df = self.df.copy()
        self.working_df["endpoint"] = 0
        self.endpoint_df = get_empty_endpoint_df()

    def run(self):
        self.process_by_ogtt_only()
        subject_identifiers_df = get_unique_subject_identifiers(self.df)
        for index, row in subject_identifiers_df.iterrows():
            subject_df = self.get_subject_df(row["subject_identifier"])
            subject_df = self.endpoint_cls(
                subject_df=subject_df,
                fbg_threshhold=self.fbg_threshhold,
                ogtt_threshhold=self.ogtt_threshhold,
            ).subject_df
            if len(subject_df.loc[subject_df["endpoint"] == 1]) == 1:
                self.append_subject_to_endpoint_df(subject_df)
                self.remove_subject_from_working_df(row)
        self.post_check_endpoint()
        self.merge_with_final_endpoints()

    def process_by_ogtt_only(self):
        """Flag subjects that met endpoint by hitting the OGTT
        threshold.

        Add them to the endpoint_df and remove them from the
        working_df.

        Subject must have fasted at the timepoint.

        The OGTT must have an FBG measure at the same timepoint.
        The value of the FBG is not considered.

        Most of these where taken off study for the OGTT. We are
        using the OGTT as the reason/date instead of the offstudy
        reason/date.

        See `merge_with_final_endpoints` where we pick the date of
        the first OGTT.
        """
        subject_endpoint_df = self.working_df.loc[
            (self.working_df["ogtt_value"] >= self.ogtt_threshhold)
            & (self.working_df["ogtt_value"] < 9999.99)
            & (self.working_df["fasted"] == YES)
            & (self.working_df["fbg_value"].notna())
        ].copy()

        subject_endpoint_df = (
            subject_endpoint_df.sort_values(by=["subject_identifier", "fbg_datetime"])
            .reset_index(drop=True)
            .drop_duplicates(subset=["subject_identifier"], keep="first")
            .reset_index(drop=True)
        )
        if not subject_endpoint_df.empty:
            # flag the selected endpoint rows as endpoints
            subject_endpoint_df["endpoint"] = 1
            subject_endpoint_df["endpoint_label"] = self.endpoint_cases[CASE_OGTT]
            subject_endpoint_df["endpoint_type"] = CASE_OGTT
            subject_endpoint_df["interval_in_days"] = np.nan

            # add back the others rows for these subjects
            subjects_df = self.working_df.loc[
                (
                    self.working_df["subject_identifier"].isin(
                        subject_endpoint_df["subject_identifier"]
                    )
                    & ~(
                        self.working_df["fbg_datetime"].isin(
                            subject_endpoint_df["fbg_datetime"]
                        )
                    )
                )
            ].copy()
            subjects_df = subjects_df.reset_index(drop=True)
            subjects_df["endpoint"] = np.nan
            subjects_df["endpoint_label"] = None
            subjects_df["endpoint_type"] = None
            subjects_df["interval_in_days"] = np.nan
            subjects_df = pd.concat([subjects_df, subject_endpoint_df])
            subjects_df = subjects_df.reset_index(drop=True)

            self.append_subject_to_endpoint_df(subjects_df[endpoint_columns])
            self.remove_subjects_from_working_df(subjects_df)

    def append_subject_to_endpoint_df(self, subject_df: pd.DataFrame) -> None:
        """Appends all rows of a subject, or subjects, to the
        Endpoints DF.
        """
        if self.endpoint_df.empty:
            self.endpoint_df = subject_df.copy()
        else:
            self.endpoint_df = pd.concat([self.endpoint_df, subject_df])
            self.endpoint_df = self.endpoint_df.sort_values(
                by=["subject_identifier", "visit_code"]
            ).reset_index(drop=True)

    def remove_subject_from_working_df(self, row: pd.Series) -> None:
        """Removes one subject from the working DF given a Series with
        value `subject_identifier`.
        """
        self.working_df = self.working_df.drop(
            index=self.working_df[
                self.working_df["subject_identifier"] == row["subject_identifier"]
            ].index
        ).reset_index(drop=True)

    def remove_subjects_from_working_df(self, rows: pd.DataFrame) -> None:
        """Removes subjects from the working DF given a DF with
        column `subject_identifier`.
        """
        self.working_df = self.working_df.drop(
            index=self.working_df.loc[
                self.working_df["subject_identifier"].isin(rows["subject_identifier"])
            ].index
        ).reset_index(drop=True)

    def get_subject_df(self, subject_identifier: str) -> pd.DataFrame:
        subject_df = self.working_df.loc[
            self.working_df["subject_identifier"] == subject_identifier
        ].copy()
        subject_df["interval_in_days"] = np.nan
        subject_df["endpoint_type"] = None
        subject_df["endpoint_label"] = None
        subject_df["endpoint"] = 0
        subject_df = subject_df[endpoint_columns]
        subject_df = subject_df.sort_values(["subject_identifier", "fbg_datetime"])
        subject_df[[col for col in subject_df if "value" in col]] = subject_df[
            [col for col in subject_df if "value" in col]
        ].fillna(0.0)

        subject_df = subject_df.reset_index(drop=True)
        return subject_df

    def check_endpoint_by_fbg_for_subject(
        self, subject_df: pd.DataFrame, case_list: list[int] | None = None
    ) -> pd.DataFrame:
        endpoint = self.endpoint_cls(
            subject_df=subject_df,
            fbg_threshhold=self.fbg_threshhold,
            ogtt_threshhold=self.ogtt_threshhold,
        )
        return endpoint.subject_df

    def post_check_endpoint(self):
        """Add any who were taken off study before endpoint guidelines
        were clearly defined.
        """
        df_eos = self.working_df.loc[
            self.working_df["offstudy_reason"] == "Patient developed diabetes"
        ].copy()
        df_eos["endpoint"] = 1
        df_eos["endpoint_label"] = self.endpoint_cases[CASE_EOS]
        df_eos["endpoint_type"] = CASE_EOS
        df_eos["interval_in_days"] = np.nan
        df_eos = df_eos.reset_index(drop=True)
        self.append_subject_to_endpoint_df(df_eos[endpoint_columns])
        self.working_df = self.working_df.drop(
            index=self.working_df.loc[
                self.working_df["subject_identifier"].isin(df_eos["subject_identifier"])
            ].index
        )

    def merge_with_final_endpoints(self):
        """Merge endpoint_df with original df"""
        if self.endpoint_df.empty:
            self.df = self.df[~(self.df["subject_identifier"].isin(self.subject_identifiers))]
        else:
            self.endpoint_df["test"] = self.endpoint_df.apply(get_test_string, axis=1)
            self.endpoint_df.loc[self.endpoint_df["endpoint"] == 1, "days_to_endpoint"] = (
                self.endpoint_df["fbg_datetime"] - self.endpoint_df["baseline_datetime"]
            ).dt.days

            # Create DF of subjects taken offstudy (EOS) where endpoint==1.
            # Keep the last record for the subject by fbg_datetime.
            df1 = self.endpoint_df.copy()
            df1 = df1[
                (df1["endpoint_type"].isin([CASE_EOS, CASE_OGTT])) & (df1["endpoint"] == 1)
            ]
            df1 = df1.sort_values(["subject_identifier", "fbg_datetime"])
            df1 = df1.reset_index(drop=True)
            df1 = df1.set_index(["subject_identifier"])
            df1 = df1[~df1.index.duplicated(keep="last")]
            df1 = df1.reset_index(drop=False)

            # Create DF of subjects still on-study where endpoint==1.
            # Keep the first record for the subject by fbg_datetime.
            df2 = self.endpoint_df.copy()
            df2 = df2[
                ~(df2["endpoint_type"].isin([CASE_EOS, CASE_OGTT])) & (df2["endpoint"] == 1)
            ]
            df2 = df2.sort_values(["subject_identifier", "fbg_datetime"])
            df2 = df2.reset_index(drop=True)
            df2 = df2.set_index(["subject_identifier"])
            df2 = df2[~df2.index.duplicated(keep="first")]
            df2 = df2.reset_index(drop=False)

            # create new DF with ONE row per subject for those that reached
            # the endpoint (endpoint=1) by merging two DFs above.
            self.endpoint_only_df = pd.concat([df1, df2])
            self.endpoint_only_df = self.endpoint_only_df.reset_index(drop=True)

            self.df = pd.merge(
                self.df,
                self.endpoint_only_df[["subject_identifier", "visit_code", "endpoint"]],
                on=["subject_identifier", "visit_code"],
                how="left",
                suffixes=("", "_y"),
            )
        self.df = self.df.sort_values(by=["subject_identifier", "fbg_datetime"])
        self.df = self.df.reset_index(drop=True)

    def to_model(self):
        """Write endpoint_only_df to the Endpoints model"""
        df = self.endpoint_only_df
        model = "meta_reports.endpoints"
        now = get_utcnow()
        model_cls = django_apps.get_model(model)
        if self.subject_identifiers:
            model_cls.objects.filter(subject_identifier__in=self.subject_identifiers).delete()
            df = (
                self.endpoint_only_df[
                    self.endpoint_only_df["subject_identifier"].isin(self.subject_identifiers)
                ]
                .copy()
                .sort_values(by=["subject_identifier"])
                .reset_index(drop=True)
            )
        else:
            model_cls.objects.all().delete()
        created = 0
        if not df.empty:
            df["fbg_datetime"] = df["fbg_datetime"].dt.tz_localize("UTC")
            df["baseline_datetime"] = df["baseline_datetime"].dt.tz_localize("UTC")
            data = [
                model_cls(
                    subject_identifier=row["subject_identifier"],
                    site_id=row["site_id"],
                    baseline_date=(
                        None if pd.isna(row["baseline_datetime"]) else row["baseline_datetime"]
                    ),
                    visit_code=None if pd.isna(row["visit_code"]) else row["visit_code"],
                    fbg_value=(None if pd.isna(row["fbg_value"]) else row["fbg_value"]),
                    ogtt_value=None if pd.isna(row["ogtt_value"]) else row["ogtt_value"],
                    fbg_date=(None if pd.isna(row["fbg_datetime"]) else row["fbg_datetime"]),
                    fasting=(None if pd.isna(row["fasted"]) else row["fasted"]),
                    endpoint_label=(
                        None if pd.isna(row["endpoint_label"]) else row["endpoint_label"]
                    ),
                    offstudy_date=(
                        None if pd.isna(row["offstudy_datetime"]) else row["offstudy_datetime"]
                    ),
                    offstudy_reason=(
                        None if pd.isna(row["offstudy_reason"]) else row["offstudy_reason"]
                    ),
                    report_model=model,
                    created=now,
                )
                for _, row in df.iterrows()
            ]
            created = len(model_cls.objects.bulk_create(data))
        return created



from .subject_consent_form import SubjectConsentForm
from .subject_consent_v1_ext_form import SubjectConsentV1ExtForm
from .subject_consent_v1_form import SubjectConsentV1Form
from .subject_reconsent_form import SubjectReconsentForm



from .blood_results_fbc_form import BloodResultsFbcForm
from .blood_results_hba1c_form import BloodResultsHba1cForm
from .blood_results_ins_form import BloodResultsInsForm
from .blood_results_lft_form import BloodResultsLftForm
from .blood_results_lipids_form import BloodResultsLipidsForm
from .blood_results_rft_form import BloodResultsRftForm



from dateutil.relativedelta import relativedelta
from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase
from edc_action_item.models import ActionItem
from edc_appointment.constants import COMPLETE_APPT
from edc_appointment.models import Appointment
from edc_constants.constants import CLOSED, FEMALE, NEW, NO, PATIENT, YES
from edc_pharmacy.constants import IN_PROGRESS_APPT
from edc_utils import get_utcnow
from edc_visit_schedule.constants import MONTH1, OFFSCHEDULE_ACTION
from edc_visit_tracking.constants import SCHEDULED

from meta_lists.models import MissedReferralReasons
from meta_prn.constants import (
    OFFSCHEDULE_DM_REFERRAL_ACTION,
    OFFSTUDY_MEDICATION_ACTION,
)
from meta_prn.models import DmReferral, OnScheduleDmReferral
from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_subject.constants import DM_FOLLOWUP_ACTION
from meta_subject.models import DmEndpoint, DmFollowup, SubjectVisit
from meta_visit_schedule.constants import DM_BASELINE, DM_FOLLOWUP, SCHEDULE_DM_REFERRAL


class TestDmReferral(MetaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_visit = self.get_subject_visit(gender=FEMALE)

    def test_dm_referral_puts_subject_on_dm_followup_schedule(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)
        dm_referral = DmReferral.objects.create(
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=get_utcnow(),
            referral_date=get_utcnow(),
        )
        self.assertIsNotNone(dm_referral.report_datetime)
        self.assertIsNotNone(dm_referral.referral_date)
        self.assertIsNotNone(dm_referral.action_identifier)

        # verify subject is on DM Followup schedule
        try:
            OnScheduleDmReferral.objects.get(
                subject_identifier=subject_visit.subject_identifier
            )
        except ObjectDoesNotExist:
            self.fail("OnScheduleDmReferral unexpectedly does not exist")

    def test_dm_referral_action_creates_offschedule_action(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)
        dm_referral = DmReferral.objects.create(
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=get_utcnow(),
            referral_date=get_utcnow(),
        )
        self.assertIsNotNone(dm_referral.report_datetime)
        self.assertIsNotNone(dm_referral.referral_date)
        self.assertIsNotNone(dm_referral.action_identifier)

        try:
            ActionItem.objects.get(
                subject_identifier=subject_visit.subject_identifier,
                action_type__name=OFFSCHEDULE_ACTION,
                status=CLOSED,
            )
        except ObjectDoesNotExist:
            self.fail(f"{OFFSCHEDULE_ACTION} Action item unexpectedly does not exist")

    def test_dm_referral_creates_offstudy_med_action(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)
        dm_referral = DmReferral.objects.create(
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=get_utcnow(),
            referral_date=get_utcnow(),
        )
        self.assertIsNotNone(dm_referral.report_datetime)
        self.assertIsNotNone(dm_referral.referral_date)
        self.assertIsNotNone(dm_referral.action_identifier)

        # verify action items are created
        try:
            ActionItem.objects.get(
                subject_identifier=subject_visit.subject_identifier,
                action_type__name=OFFSTUDY_MEDICATION_ACTION,
                status=NEW,
            )
        except ObjectDoesNotExist:
            self.fail(f"{OFFSTUDY_MEDICATION_ACTION} Action item unexpectedly does not exist")

    def test_dm_referral_creates_dm_followup_action(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)
        dm_referral = DmReferral.objects.create(
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=get_utcnow(),
            referral_date=get_utcnow(),
        )
        self.assertIsNotNone(dm_referral.report_datetime)
        self.assertIsNotNone(dm_referral.referral_date)
        self.assertIsNotNone(dm_referral.action_identifier)

        try:
            ActionItem.objects.get(
                subject_identifier=subject_visit.subject_identifier,
                action_type__name=DM_FOLLOWUP_ACTION,
                status=NEW,
            )
        except ObjectDoesNotExist:
            self.fail(f"{DM_FOLLOWUP_ACTION} Action item unexpectedly does not exist")

    def test_dm_referral2(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)
        referral_datetime = subject_visit.report_datetime
        DmReferral.objects.create(
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=referral_datetime,
            referral_date=referral_datetime.date(),
        )

        # Add DM Baseline
        appointment = Appointment.objects.get(
            subject_identifier=subject_visit.subject_identifier,
            schedule_name=SCHEDULE_DM_REFERRAL,
            visit_code=DM_BASELINE,
        )
        appointment.appt_status = IN_PROGRESS_APPT
        appointment.save()

        subject_visit = SubjectVisit.objects.create(
            appointment=appointment,
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=appointment.appt_datetime,
            reason=SCHEDULED,
            info_source=PATIENT,
        )

        DmEndpoint.objects.create(
            subject_visit=subject_visit,
            report_datetime=get_utcnow(),
            dx_date=referral_datetime.date(),
            dx_initiated_by="fbg_confirmed",
            dx_tmg=YES,
            dx_tmg_date=referral_datetime.date(),
        )

        # Add DM Followup
        followup_datetime = referral_datetime + relativedelta(months=6)
        appointment = Appointment.objects.get(
            subject_identifier=subject_visit.subject_identifier,
            schedule_name=SCHEDULE_DM_REFERRAL,
            visit_code=DM_FOLLOWUP,
        )
        appointment.appt_status = IN_PROGRESS_APPT
        appointment.save()

        subject_visit = SubjectVisit.objects.create(
            appointment=appointment,
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=appointment.appt_datetime,
            reason=SCHEDULED,
            info_source=PATIENT,
        )

        dm_followup = DmFollowup.objects.create(
            subject_visit=subject_visit,
            report_datetime=followup_datetime,
            referral_date=referral_datetime.date(),
            attended=NO,
            on_dm_medications=NO,
        )
        dm_followup.missed_referral_reasons.set([MissedReferralReasons.objects.all()[0]])

        appointment.appt_status = COMPLETE_APPT
        appointment.save()

        try:
            ActionItem.objects.get(
                subject_identifier=subject_visit.subject_identifier,
                action_type__name=DM_FOLLOWUP_ACTION,
                status=CLOSED,
            )
        except ObjectDoesNotExist:
            self.fail(f"{DM_FOLLOWUP_ACTION} Action item unexpectedly does not exist")

        try:
            ActionItem.objects.get(
                subject_identifier=subject_visit.subject_identifier,
                action_type__name=OFFSCHEDULE_DM_REFERRAL_ACTION,
                status=NEW,
            )
        except ObjectDoesNotExist:
            self.fail(
                f"{OFFSCHEDULE_DM_REFERRAL_ACTION} Action item unexpectedly does not exist"
            )



import numpy as np
import pandas as pd
from django_pandas.io import read_frame

from meta_screening.models import SubjectScreening
from meta_subject.models import PhysicalExam, SubjectVisit


def get_screening_df(df: pd.DataFrame | None = None) -> pd.DataFrame:
    df = pd.DataFrame() if not hasattr(df, "empty") else df
    if df.empty:
        cols = [
            f.name
            for f in SubjectScreening._meta.get_fields()
            if f.name
            not in [
                "contact_number",
                "initials",
                "hospital_identifier",
                "modified",
                "user_created",
                "user_modified",
                "hostname_created",
                "hostname_modified",
                "device_created",
                "device_modified",
                "locale_created",
                "locale_modified",
                "slug",
            ]
        ]
        qs_screening = SubjectScreening.objects.values(*cols).all()
        df = read_frame(qs_screening, verbose=False)
        df = df.drop(df[df["hiv_pos"] == "No"].index)  # removes 2 rows
        df = df.reset_index(drop=True)

    # convert all to float
    num_cols = [
        "age_in_years",
        "calculated_bmi_value",
        "converted_fbg2_value",
        "converted_fbg_value",
        "converted_ogtt2_value",
        "converted_ogtt_value",
        "dia_blood_pressure_avg",
        "fbg2_value",
        "fbg_value",
        "hba1c_value",
        "ogtt2_value",
        "ogtt_value",
        "sys_blood_pressure_avg",
        "waist_circumference",
    ]
    df[num_cols] = df[num_cols].apply(pd.to_numeric)

    df["reasons_ineligible_part_one"] = df["reasons_ineligible_part_one"].apply(
        lambda x: None if x == "" else x
    )
    df["reasons_ineligible_part_two"] = df["reasons_ineligible_part_two"].apply(
        lambda x: None if x == "" else x
    )
    df["reasons_ineligible_part_two"] = df["reasons_ineligible_part_two"].str.replace(
        "Has Dm", "Diabetes"
    )
    df["reasons_ineligible_part_two"] = df["reasons_ineligible_part_two"].str.replace(
        "On Dm Medication", "taking anti-diabetic medications"
    )
    df["reasons_ineligible_part_three"] = df["reasons_ineligible_part_three"].apply(
        lambda x: None if x == "" else x
    )

    # condition to include any glucose test

    # has_dm fillna with unk
    df["has_dm"] = df["has_dm"].apply(lambda x: "unk" if not x else x)

    na = "Not applicable, subject is not eligible based on the criteria above"
    df["already_fasted"] = df["already_fasted"].apply(lambda x: "N/A" if x == na else x)

    # create a column that summarizes lives_nearby and staying_nearby_12
    df["in_catchment"] = (df["lives_nearby"] == "Yes") & (df["staying_nearby_12"] == "Yes")

    # create ogtt column
    df["ogtt"] = df["converted_ogtt_value"]
    df.loc[df["ogtt"].notna() & df["converted_ogtt2_value"].notna(), "ogtt"] = df[
        "converted_ogtt2_value"
    ]

    # create fbg column
    df["fbg"] = df["converted_fbg_value"]
    df.loc[df["fbg"].notna() & df["converted_fbg2_value"].notna(), "fbg"] = df[
        "converted_fbg2_value"
    ]

    # fasting columns
    df["fasting_fbg_hrs"] = np.nan
    df["fasting_fbg_hrs"] = df["fasting_duration_delta"].apply(
        lambda x: x.total_seconds() / 3600
    )
    df.loc[df["fbg"].notna() & df["converted_fbg2_value"].notna(), "fasting_fbg_hrs"] = df[
        "repeat_fasting_duration_delta"
    ].apply(lambda x: x.total_seconds() / 3600)
    df["fasting_ogtt_hrs"] = np.nan
    df["fasting_ogtt_hrs"] = df["fasting_duration_delta"].apply(
        lambda x: x.total_seconds() / 3600
    )
    df.loc[df["ogtt"].notna() & df["converted_ogtt2_value"].notna(), "fasting_ogtt_hrs"] = df[
        "repeat_fasting_duration_delta"
    ].apply(lambda x: x.total_seconds() / 3600)

    # bmi
    # subject SR9E8B4D has eligible part two == No but subject has a glucose value
    # NOTE: update 10-02-2025: subject is not eligible by congestive_heart_failure
    # df.loc[(df["screening_identifier"] == "SR9E8B4D"), "eligible_part_two"] = "Yes"

    # merge with physical exam to get waist circumference if taken at baseline
    subject_identifiers = list(df["subject_identifier"])
    qs_subject_visit = SubjectVisit.objects.filter(subject_identifier__in=subject_identifiers)
    df_subject_visit = read_frame(qs_subject_visit)
    df_subject_visit.rename(columns={"id": "subject_visit"}, inplace=True)
    qs_physical_exam = PhysicalExam.objects.filter(
        subject_visit__subject_identifier__in=subject_identifiers
    )
    df_physical_exam = read_frame(qs_physical_exam)
    # merge w/ subject visit to get subject_identifier
    df_physical_exam = pd.merge(
        df_physical_exam,
        df_subject_visit[
            ["subject_visit", "subject_identifier", "visit_code", "visit_code_sequence"]
        ],
        on="subject_visit",
        how="left",
    )
    df_physical_exam = df_physical_exam[
        ["subject_identifier", "visit_code", "visit_code_sequence", "waist_circumference"]
    ]
    df_physical_exam[["waist_circumference"]] = df_physical_exam[
        ["waist_circumference"]
    ].apply(pd.to_numeric)
    # rename column to waist_circumference_baseline
    df_physical_exam["waist_circumference_baseline"] = df_physical_exam["waist_circumference"]
    df_physical_exam.drop(columns=["waist_circumference"])
    df_physical_exam[["waist_circumference_baseline"]] = df_physical_exam[
        ["waist_circumference_baseline"]
    ].apply(pd.to_numeric)
    # merge on subject_identifier with main DF
    df = pd.merge(
        df,
        df_physical_exam[["subject_identifier", "waist_circumference_baseline"]],
        on="subject_identifier",
        how="left",
    )
    df.reset_index(drop=True, inplace=True)
    # set waist_circumference=waist_circumference_baseline
    # if `waist_circumference` is none and `waist_circumference_baseline` is not
    df.loc[
        (df["waist_circumference"].isna()) & (df["waist_circumference_baseline"].notna()),
        "waist_circumference",
    ] = df["waist_circumference_baseline"]

    # drop waist_circumference_baseline
    df.drop(columns=["waist_circumference_baseline"], inplace=True)

    return df



from django.apps import AppConfig as DjangoAppConfig


class AppConfig(DjangoAppConfig):
    name = "meta_labs"
    default_auto_field = "django.db.models.BigAutoField"
    verbose_name = "META Labs"



from django.contrib.admin import SimpleListFilter
from edc_constants.choices import YES_NO_NA
from edc_constants.constants import NO, NOT_APPLICABLE, YES


class AgreesListFilter(SimpleListFilter):
    title = "Agrees"
    parameter_name = "agrees"

    def lookups(self, request, model_admin):
        return YES_NO_NA

    def queryset(self, request, queryset):
        qs = None
        if self.value():
            if self.value() == YES:
                qs = queryset.filter(agrees_to_extension=YES)
            elif self.value() == NO:
                qs = queryset.filter(agrees_to_extension=NO)
            elif self.value() == NOT_APPLICABLE:
                qs = queryset.filter(agrees_to_extension=NOT_APPLICABLE)
        return qs



import pandas as pd
from edc_pdutils.dataframes import get_eos, get_subject_visit


def get_eos_df() -> pd.DataFrame:
    """
    df = get_eos_df()

    # look at transfers and last attended visit
    df[(df.transfer_reason.notna())]

    """
    df_eos = get_eos("meta_prn.endofstudy")
    df_visit = get_subject_visit("meta_subject.subjectvisit")
    df_last_visit = (
        df_visit.groupby(["subject_identifier", "site_id"])
        .agg({"endline_visit_code": "max", "endline_visit_datetime": "max"})
        .reset_index()
    )
    # df_last_visit = df_last_visit.rename(columns={"site": "site_id"})

    df_eos = df_eos.merge(
        df_last_visit, on="subject_identifier", how="left", suffixes=("", "_y")
    )
    df_eos = df_eos.drop(columns=["site_id_y"])
    df_visit_grp = (
        df_visit.groupby(by=["subject_identifier"])[["baseline_datetime", "visit_datetime"]]
        .max()
        .reset_index()
    )
    df_visit_grp["followup_days"] = (
        df_visit_grp["visit_datetime"] - df_visit_grp["baseline_datetime"]
    ).dt.days
    df_eos = df_eos.merge(
        df_visit_grp[["subject_identifier", "followup_days"]],
        on="subject_identifier",
        how="left",
    ).reset_index(drop=True)
    return df_eos



from __future__ import annotations

from django.contrib import admin
from edc_appointment.modeladmin_mixins import NextAppointmentCrfModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_subject_admin
from ..forms import NextAppointmentForm
from ..models import NextAppointment
from .modeladmin import CrfModelAdminMixin

__all__ = ["NextAppointmentAdmin"]


@admin.register(NextAppointment, site=meta_subject_admin)
class NextAppointmentAdmin(
    CrfModelAdminMixin, NextAppointmentCrfModelAdminMixin, SimpleHistoryAdmin
):

    form = NextAppointmentForm
